package sun.management.snmp.jvminstr;

import sun.management.snmp.util.SnmpCachedData;
import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import sun.management.snmp.util.JvmContextFactory;
import sun.management.snmp.util.SnmpTableHandler;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpOid;
import com.sun.jmx.snmp.agent.SnmpStandardObjectServer;
import com.sun.jmx.snmp.agent.SnmpMib;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.util.SnmpTableCache;
import sun.management.snmp.jvmmib.JvmRTClassPathTableMeta;

public class JvmRTClassPathTableMetaImpl extends JvmRTClassPathTableMeta
{
    static final long serialVersionUID = -6914494148818455166L;
    private SnmpTableCache cache;
    static final MibLogger log;
    
    public JvmRTClassPathTableMetaImpl(SnmpMib snmpMib, SnmpStandardObjectServer snmpStandardObjectServer) {
        super(snmpMib, snmpStandardObjectServer);
        this.cache = new JvmRTClassPathTableMetaImpl.JvmRTClassPathTableCache(this, -1L);
    }
    
    protected SnmpOid getNextOid(Object o) throws SnmpStatusException {
        return this.getNextOid(null, o);
    }
    
    protected SnmpOid getNextOid(SnmpOid obj, Object o) throws SnmpStatusException {
        int debugOn = JvmRTClassPathTableMetaImpl.log.isDebugOn();
        if (!(debugOn == 0)) {
            JvmRTClassPathTableMetaImpl.log.debug("getNextOid", "previous=" + obj);
        }
        SnmpTableHandler handler = this.getHandler(o);
        if (handler == null) {
            if (!(debugOn == 0)) {
                JvmRTClassPathTableMetaImpl.log.debug("getNextOid", "handler is null!");
            }
            throw new SnmpStatusException(224);
        }
        else {
            SnmpOid next = handler.getNext(obj);
            if (!(debugOn == 0)) {
                JvmRTClassPathTableMetaImpl.log.debug("*** **** **** **** getNextOid", "next=" + next);
            }
            if (next == null) {
                throw new SnmpStatusException(224);
            }
            return next;
        }
    }
    
    protected boolean contains(SnmpOid snmpOid, Object o) {
        SnmpTableHandler handler = this.getHandler(o);
        return handler != null && handler.contains(snmpOid);
    }
    
    public Object getEntry(SnmpOid snmpOid) throws SnmpStatusException {
        int debugOn = JvmRTClassPathTableMetaImpl.log.isDebugOn();
        if (!(debugOn == 0)) {
            JvmRTClassPathTableMetaImpl.log.debug("getEntry", "oid [" + snmpOid + "]");
        }
        if (snmpOid == null || snmpOid.getLength() != 1) {
            if (!(debugOn == 0)) {
                JvmRTClassPathTableMetaImpl.log.debug("getEntry", "Invalid oid [" + snmpOid + "]");
            }
            throw new SnmpStatusException(224);
        }
        else {
            Map<String, JvmRTClassPathEntryImpl> userData = JvmContextFactory.getUserData();
            String s = (userData == null) ? null : ("JvmRTClassPathTable.entry." + snmpOid.toString());
            if (userData != null) {
                Object value = userData.get(s);
                if (value != null) {
                    if (!(debugOn == 0)) {
                        JvmRTClassPathTableMetaImpl.log.debug("getEntry", "Entry is already in the cache");
                        return value;
                    }
                    return value;
                }
                else if (!(debugOn == 0)) {
                    JvmRTClassPathTableMetaImpl.log.debug("getEntry", "Entry is not in the cache");
                }
            }
            SnmpTableHandler handler = this.getHandler(userData);
            if (handler == null) {
                throw new SnmpStatusException(224);
            }
            Object data = handler.getData(snmpOid);
            if (data == null) {
                throw new SnmpStatusException(224);
            }
            if (!(debugOn == 0)) {
                JvmRTClassPathTableMetaImpl.log.debug("getEntry", "data is a: " + data.getClass().getName());
            }
            JvmRTClassPathEntryImpl jvmRTClassPathEntryImpl = new JvmRTClassPathEntryImpl((String)data, (int)snmpOid.getOidArc(0));
            if (userData != null && jvmRTClassPathEntryImpl != null) {
                userData.put(s, jvmRTClassPathEntryImpl);
                return jvmRTClassPathEntryImpl;
            }
            return jvmRTClassPathEntryImpl;
        }
    }
    
    protected SnmpTableHandler getHandler(Object x) {
        Map<String, SnmpTableHandler> map;
        if ((x instanceof Map)) {
            map = (Map)Util.cast(x);
        }
        else {
            map = null;
        }
        if (map != null) {
            SnmpTableHandler snmpTableHandler = (SnmpTableHandler)map.get("JvmRTClassPathTable.handler");
            if (snmpTableHandler != null) {
                return snmpTableHandler;
            }
        }
        SnmpTableHandler tableHandler = this.cache.getTableHandler();
        if (map != null && tableHandler != null) {
            map.put("JvmRTClassPathTable.handler", tableHandler);
            return tableHandler;
        }
        return tableHandler;
    }
    
    static {
        log = new MibLogger(JvmRTClassPathTableMetaImpl.class);
    }
    
    private static class JvmRTClassPathTableCache extends SnmpTableCache
    {
        static final long serialVersionUID = 3805032372592117315L;
        private JvmRTClassPathTableMetaImpl meta;
        
        JvmRTClassPathTableCache(JvmRTClassPathTableMetaImpl meta, long validity) {
            this.meta = meta;
            this.validity = validity;
        }
        
        public SnmpTableHandler getTableHandler() {
            return this.getTableDatas(JvmContextFactory.getUserData());
        }
        
        protected SnmpCachedData updateCachedDatas(Object o) {
            String[] classPath = JvmRuntimeImpl.getClassPath(o);
            long currentTimeMillis = System.currentTimeMillis();
            int length = classPath.length;
            SnmpOid[] array = new SnmpOid[length];
            for (int i = 0; i < length; ++i) {
                array[i] = new SnmpOid((long)(i + 1));
            }
            return new SnmpCachedData(currentTimeMillis, array, classPath);
        }
    }
}
